@@ -84,9 +84,8 @@ impl_Bonobo_Embeddable_getClientSite (PortableServer_Servant servant,
 	BonoboEmbeddable *embeddable = BONOBO_EMBEDDABLE (
 		bonobo_object_from_servant (servant));
 
-	return bonobo_object_dup_ref (
-		bonobo_object_corba_objref (
-			BONOBO_OBJECT (embeddable->client_site)), ev);
+	return bonobo_object_dup_ref (BONOBO_OBJREF (embeddable->client_site),
+				      ev);
 }
 
 static void
@@ -202,7 +201,7 @@ impl_Bonobo_Embeddable_createView (PortableServer_Servant servant,
 	if (view == NULL)
 		return CORBA_OBJECT_NIL;
 
-	if (bonobo_object_corba_objref (BONOBO_OBJECT (view)) == CORBA_OBJECT_NIL){
+	if (BONOBO_OBJREF (view) == CORBA_OBJECT_NIL){
 		g_warning ("Returned view does not have a CORBA object bound");
 		bonobo_object_unref (BONOBO_OBJECT (view));
 		return CORBA_OBJECT_NIL;
@@ -215,8 +214,7 @@ impl_Bonobo_Embeddable_createView (PortableServer_Servant servant,
 	gtk_signal_connect (GTK_OBJECT (view), "destroy",
 			    GTK_SIGNAL_FUNC (bonobo_embeddable_view_destroy_cb), embeddable);
 
-	return CORBA_Object_duplicate (
-		bonobo_object_corba_objref (BONOBO_OBJECT (view)), ev);
+	return CORBA_Object_duplicate (BONOBO_OBJREF (view), ev);
 }
 
 static void
@@ -251,11 +249,7 @@ make_canvas_component (BonoboEmbeddable *embeddable, gboolean aa, Bonobo_Canvas_
 	BonoboCanvasComponent *component;
 	GnomeCanvas *pseudo_canvas;
 	
-	if (aa){
-		gdk_rgb_init ();
-		pseudo_canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	} else
-		pseudo_canvas = GNOME_CANVAS (gnome_canvas_new ());
+	pseudo_canvas = bonobo_canvas_new (aa, item_proxy);
 	
 	component = (*embeddable->priv->item_creator)(
 		embeddable, pseudo_canvas,
@@ -265,11 +259,8 @@ make_canvas_component (BonoboEmbeddable *embeddable, gboolean aa, Bonobo_Canvas_
 		gtk_object_destroy (GTK_OBJECT (pseudo_canvas));
 		return NULL;
 	}
-	bonobo_canvas_component_set_proxy (component, item_proxy);
 
-	/*
-	 * Now keep track of it
-	 */
+	/* Now keep track of it */
 	embeddable->priv->canvas_items = g_list_prepend (embeddable->priv->canvas_items, component);
 	gtk_signal_connect (GTK_OBJECT (component), "destroy",
 			    GTK_SIGNAL_FUNC (canvas_item_destroyed), embeddable);
@@ -294,8 +285,7 @@ impl_Bonobo_Embeddable_createCanvasItem (PortableServer_Servant servant,
 	
 	component = make_canvas_component (embeddable, aa, item_proxy);
 
-	return bonobo_object_dup_ref (
-		bonobo_object_corba_objref (BONOBO_OBJECT (component)), ev);
+	return bonobo_object_dup_ref (BONOBO_OBJREF (component), ev);
 }
 
 /**
