@@ -8,6 +8,8 @@
  * Copyright (C) 2000, Helix Code, Inc.
  */
 #include <config.h>
+#include <stdio.h>
+#include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 
 #include <bonobo/bonobo-context.h>
@@ -16,6 +18,18 @@
 #include <bonobo/bonobo-moniker-util.h>
 #include <bonobo/bonobo-running-context.h>
 
+#ifdef BONOBO_OBJECT_DEBUG
+#	define BONOBO_RUNNING_HOOKS
+#endif
+
+/*
+ * NB. for a quicker debugging experience simply
+ * #define BONOBO_RUNNING_HOOKS
+ */
+#if 0
+#	define BONOBO_RUNNING_HOOKS
+#endif
+
 POA_Bonobo_RunningContext__vepv bonobo_running_context_vepv;
 
 typedef struct {
@@ -41,12 +55,49 @@ key_free (gpointer name, gpointer dummy1, gpointer user_data)
 	g_free (name);
 }
 
+#ifdef BONOBO_RUNNING_HOOKS
+static void
+bonobo_debug_print (char *name, char *fmt, ...)
+{
+	va_list args;
+           
+	va_start (args, fmt);
+	
+	printf ("[%06d]:%-15s ", getpid (), name); 
+	vprintf (fmt, args);
+	printf ("\n"); 
+
+	va_end (args);
+}
+
+static void
+bonobo_ri_debug_foreach (gpointer key, gpointer value, gpointer user_data)
+{
+	CORBA_Object *o = value;
+	
+	bonobo_debug_print ("", "[%p]:CORBA_Object still running", o);
+		
+}
+#endif
+
 static void
 running_info_destroy (void)
 {
 	if (bonobo_running_info) {
+
 		BonoboRunningInfo *ri = bonobo_running_info;
 
+#ifdef BONOBO_RUNNING_HOOKS
+		bonobo_debug_print ("rinfo-start", 
+			  "-------------------------------------------------");
+
+		bonobo_debug_print ("running-objects", "%d running objects", 
+				    g_hash_table_size (ri->objects));
+		g_hash_table_foreach (ri->objects,
+				      bonobo_ri_debug_foreach, NULL);
+		bonobo_debug_print ("rinfo-end", 
+			  "-------------------------------------------------");
+#endif
 		if (ri->objects)
 			g_hash_table_destroy (ri->objects);
 		ri->objects = NULL;
@@ -82,6 +133,7 @@ get_running_info (gboolean create)
 		bonobo_running_info = g_new (BonoboRunningInfo, 1);
 		bonobo_running_info->objects = g_hash_table_new (NULL, NULL);
 		bonobo_running_info->keys    = g_hash_table_new (g_str_hash, g_str_equal);
+		bonobo_running_info->emitted_last_unref = FALSE;
 
 		g_atexit (running_info_destroy);
 	}
@@ -114,17 +166,28 @@ check_empty (void)
 	}
 }
 
+#ifndef bonobo_running_context_add_object
 void
 bonobo_running_context_add_object (CORBA_Object object)
 {
+#ifdef BONOBO_RUNNING_HOOKS
+	bonobo_running_context_trace_objects (object, "local", 0, 0);
+#else 
 	BonoboRunningInfo *ri = get_running_info (TRUE);
 
 	g_hash_table_insert (ri->objects, object, object);
+#endif
 }
+#endif
 
+
+#ifndef bonobo_running_context_remove_object
 void
 bonobo_running_context_remove_object (CORBA_Object object)
 {
+#ifdef BONOBO_RUNNING_HOOKS
+	bonobo_running_context_trace_objects (object, "local", 0, 1);
+#else 
 	BonoboRunningInfo *ri = get_running_info (FALSE);
 
 	if (ri) {
@@ -132,15 +195,60 @@ bonobo_running_context_remove_object (CORBA_Object object)
 
 		check_empty ();
 	}
+#endif
 }
+#endif
 
+#ifndef bonobo_running_context_ignore_object
 void
 bonobo_running_context_ignore_object (CORBA_Object object)
 {
+#ifdef BONOBO_RUNNING_HOOKS
+	bonobo_running_context_trace_objects (object, "local", 0, 2);
+#else 
 	BonoboRunningInfo *ri = get_running_info (FALSE);
 
-	if (ri)
+	if (ri) {
 		g_hash_table_remove (ri->objects, object);
+	}
+#endif
+}
+#endif
+
+void          
+bonobo_running_context_trace_objects (CORBA_Object object,
+				      const char  *fn,
+				      int          line,
+				      int          mode)
+{
+	BonoboRunningInfo *ri = get_running_info (mode == 0);
+#ifdef BONOBO_RUNNING_HOOKS
+	char *cmode[] = {
+		"add_object",
+		"remove_object",
+		"ignore_object"		
+	};
+#endif
+	if (ri) {
+		switch (mode) {
+		case 0:
+			g_hash_table_insert (ri->objects, object, object);
+			break;
+		case 1:
+			g_hash_table_remove (ri->objects, object);
+
+			check_empty ();
+			break;
+		case 2:
+			g_hash_table_remove (ri->objects, object);
+			break;
+		}
+#ifdef BONOBO_RUNNING_HOOKS
+		bonobo_debug_print (cmode [mode], 
+		        "[%p]:CORBA_Object %d running objects at %s:%d",
+			object, g_hash_table_size (ri->objects), fn, line);
+#endif
+	}
 }
 
 static void
@@ -196,6 +304,14 @@ impl_Bonobo_RunningContext_removeKey (PortableServer_Servant servant,
 	check_empty ();
 }
 
+static void
+impl_Bonobo_RunningContext_atExitUnref (PortableServer_Servant servant,
+					const CORBA_Object     object,
+					CORBA_Environment     *ev)
+{
+	bonobo_running_context_at_exit_unref (object);
+}
+
 /**
  * bonobo_running_context_get_epv:
  *
@@ -208,10 +324,11 @@ bonobo_running_context_get_epv (void)
 
 	epv = g_new0 (POA_Bonobo_RunningContext__epv, 1);
 
-	epv->addObject    = impl_Bonobo_RunningContext_addObject;
-	epv->removeObject = impl_Bonobo_RunningContext_removeObject;
-	epv->addKey       = impl_Bonobo_RunningContext_addKey;
-	epv->removeKey    = impl_Bonobo_RunningContext_removeKey;
+	epv->addObject     = impl_Bonobo_RunningContext_addObject;
+	epv->removeObject  = impl_Bonobo_RunningContext_removeObject;
+	epv->addKey        = impl_Bonobo_RunningContext_addKey;
+	epv->removeKey     = impl_Bonobo_RunningContext_removeKey;
+	epv->atExitUnref   = impl_Bonobo_RunningContext_atExitUnref;
 
 	return epv;
 }
@@ -313,8 +430,7 @@ bonobo_running_context_new (void)
 
 	bonobo_running_event_source = bonobo_event_source_new ();
 	bonobo_running_context_ignore_object (
-		bonobo_object_corba_objref (BONOBO_OBJECT (
-			bonobo_running_event_source)));
+	        BONOBO_OBJREF (bonobo_running_event_source));
 	bonobo_event_source_ignore_listeners (bonobo_running_event_source);
 
 	bonobo_object_add_interface (BONOBO_OBJECT (bonobo_running_context),
@@ -331,3 +447,52 @@ bonobo_context_running_get (void)
 {
 	return bonobo_running_context_new ();
 }
+
+static void
+last_unref_cb (gpointer      context,
+	       CORBA_Object  object)
+{
+	bonobo_object_release_unref (object, NULL);
+}
+
+void 
+bonobo_running_context_at_exit_unref (CORBA_Object object)
+{
+	CORBA_Environment ev;
+	CORBA_Object obj_dup;
+
+	CORBA_exception_init (&ev);
+
+	obj_dup = CORBA_Object_duplicate (object, &ev);
+
+	bonobo_running_context_ignore_object (obj_dup);
+
+	if (bonobo_running_context)
+		gtk_signal_connect (GTK_OBJECT (bonobo_running_context),
+				    "last_unref", last_unref_cb, obj_dup);
+	
+	CORBA_exception_free (&ev);
+}
+
+static void
+last_unref_exit_cb (gpointer      context,
+		    BonoboObject *object)
+{
+        bonobo_object_unref (object);
+	gtk_main_quit ();
+}
+
+void 
+bonobo_running_context_auto_exit_unref (BonoboObject *object)
+{
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (BONOBO_IS_OBJECT (object));
+
+	bonobo_running_context_ignore_object (BONOBO_OBJREF (object));
+
+	if (bonobo_running_context)
+		gtk_signal_connect (GTK_OBJECT (bonobo_running_context),
+				    "last_unref", last_unref_exit_cb, object);
+
+}
+
