@@ -12,6 +12,7 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
 #include <gtk/gtkplug.h>
+#include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-view.h>
 #include <gdk/gdkprivate.h>
@@ -69,7 +70,7 @@ bonobo_view_corba_object_create (BonoboObject *object)
 
 	CORBA_exception_init (&ev);
 	POA_Bonobo_View__init ((PortableServer_Servant) servant, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION){
+	if (BONOBO_EX (&ev)){
 
 		g_free (servant);
 		CORBA_exception_free (&ev);
