@@ -197,7 +197,7 @@ bonobo_widget_construct_control (BonoboWidget      *bw,
 		return NULL;
 	}
 
-	control = bonobo_object_corba_objref (BONOBO_OBJECT (bw->priv->server));
+	control = BONOBO_OBJREF (bw->priv->server);
 
 	return bonobo_widget_construct_control_from_objref (bw, control, uic);
 }
@@ -317,13 +317,6 @@ bonobo_widget_create_subdoc_object (BonoboWidget      *bw,
 	if (!bonobo_client_site_bind_embeddable (bw->priv->client_site, bw->priv->server))
 		return NULL;
 
-	/*
-	 * Add the client site to the container.  This container is
-	 * basically just there as a place holder; this is the only
-	 * client site that will ever be added to it.
-	 */
-	bonobo_item_container_add (bw->priv->container, BONOBO_OBJECT (bw->priv->client_site));
-
 	/*
 	 * Now create a new view for the remote object.
 	 */
